/*
 * This file is part of CycloneDX Rust Cargo.
 *
 * Licensed under the Apache License, Version 2.0 (the "License");
 * you may not use this file except in compliance with the License.
 * You may obtain a copy of the License at
 *
 *     http://www.apache.org/licenses/LICENSE-2.0
 *
 * Unless required by applicable law or agreed to in writing, software
 * distributed under the License is distributed on an "AS IS" BASIS,
 * WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
 * See the License for the specific language governing permissions and
 * limitations under the License.
 *
 * SPDX-License-Identifier: Apache-2.0
 */

pub(crate) mod advisory;
pub(crate) mod attached_text;
pub(crate) mod bom;
pub(crate) mod bom_reference;
pub(crate) mod code;
pub(crate) mod component;
pub(crate) mod composition;
pub(crate) mod dependency;
pub(crate) mod external_reference;
pub(crate) mod hash;
pub(crate) mod license;
pub(crate) mod metadata;
pub(crate) mod organization;
pub(crate) mod property;
pub(crate) mod service;
pub(crate) mod signature;
pub(crate) mod tool;
pub(crate) mod vulnerability;
pub(crate) mod vulnerability_analysis;
pub(crate) mod vulnerability_credits;
pub(crate) mod vulnerability_rating;
pub(crate) mod vulnerability_reference;
pub(crate) mod vulnerability_source;
pub(crate) mod vulnerability_target;
