/*
 * This file is part of CycloneDX Rust Cargo.
 *
 * Licensed under the Apache License, Version 2.0 (the "License");
 * you may not use this file except in compliance with the License.
 * You may obtain a copy of the License at
 *
 *     http://www.apache.org/licenses/LICENSE-2.0
 *
 * Unless required by applicable law or agreed to in writing, software
 * distributed under the License is distributed on an "AS IS" BASIS,
 * WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
 * See the License for the specific language governing permissions and
 * limitations under the License.
 *
 * SPDX-License-Identifier: Apache-2.0
 */

pub mod advisory;
pub mod annotation;
pub mod attached_text;
pub mod attachment;
pub mod bom;
pub mod code;
pub mod component;
pub mod component_data;
pub mod composition;
pub mod data_governance;
pub mod dependency;
pub mod external_reference;
pub mod formulation;
pub mod hash;
pub mod license;
pub mod lifecycle;
pub mod metadata;
pub mod modelcard;
pub mod organization;
pub mod property;
pub mod service;
pub mod signature;
pub mod tool;
pub mod vulnerability;
pub mod vulnerability_analysis;
pub mod vulnerability_credits;
pub mod vulnerability_rating;
pub mod vulnerability_reference;
pub mod vulnerability_source;
pub mod vulnerability_target;
